#pragma once
#include "packet.h"
#include <math.h>
#define _USE_MATH_DEFINES // To access M_PI constant for trig functions

extern packet_t pA, pB, safe;
extern packet_t *astate, *incoming;
extern comm_state cs;
extern char comm_ok;
extern long last_p;

/*
#define CLOCKWISE 0
#define ANTICLOCKWISE 1
*/

// Loop timing
#define LOOP_DELAY_MS_CORE_0    50 // Minimum milliseconds between the start of each loop, accounting for processing time during each loop
#define LOOP_DELAY_MS_CORE_1    20 // Minimum milliseconds between the start of each loop, accounting for processing time during each loop
#define LOOP_DELAY_SECONDS ((float)LOOP_DELAY_MS_CORE_0 / 1000.0f) // Previous number expressed in seconds

// Math things
#define DEGREES_PER_RADIAN (180.0 / M_PI)
#define RADIANS_PER_DEGREE (M_PI / 180.0)
#define max(x,y) ( (x) > (y) ? (x) : (y) )
#define min(x,y) ( (x) < (y) ? (x) : (y) )

#define MOTOR_MAX_POWER         127.0   // Highest value accepted by motor control functions
#define DRIVE_MOTOR_MAX_POWER   64.0    // Maximum power for drive motors 

// Drive modes
#define DRIVE_STOP          0
#define DRIVE_BASIC         1
#define DRIVE_TRANSLATION   2
#define DRIVE_ROTATION      3
#define DRIVE_TANK          4

// Default drive mode, currently default to stopping upon initialization
#define DEFAULT_SWERVE_DRIVE_MODE DRIVE_STOP

// Whether to enable swerve drive steering by default (set this to false if the robot is locked to tank drive)
#define DEFAULT_SWERVE_ENABLE_STEERING true

// Controller maximum inputs for joystick
#define CONTROLLER_JOYSTICK_MAX 128.0

// Basic mode conditions, specifies which direction and turning direction the robot is using
#define DRIVE_BASIC_STOP        0
#define DRIVE_BASIC_FORWARD     1
#define DRIVE_BASIC_FRONTLEFT   2
#define DRIVE_BASIC_FRONTRIGHT  3
#define DRIVE_BASIC_BACKWARD    4
#define DRIVE_BASIC_BACKLEFT    5
#define DRIVE_BASIC_BACKRIGHT   6

// Length of the buffer to monitor recent steering encoder positions to calculate speed
// The buffer will track the last N states of the encoders, and the times at which they were recorded, to determine the steering motors' current speeds
// This value must always be at least 2, otherwise the code will break due to there being an array with a zero or negative length or a division by zero
#define ENCODER_BUFFER_ENTRY_COUNT 3

// Steering parameters
#define STEERING_ENCODER_TICKS_PER_ROTATION     (1024.0 * 8.0)                                  // Number of encoder ticks per full rotation of each swerve drive steering motor
#define STEERING_ENCODER_TICKS_PER_DEGREE       (STEERING_ENCODER_TICKS_PER_ROTATION / 360.0)   // Number of encoder ticks per degree of rotation for the swerve drive steering motors
#define STEERING_MOTOR_SPEED_LIMIT 80.0                                                         // Maximum speed allowed for the steering motors (out of 127.0)
// Steering PID parameters
#define STEERING_SLOW_DELTA 35.0                                                                // Start decelerating the steering motors linearly when they're within this many degrees of their target angle
#define STEERING_ACCEL_SLOW_DELAY 0.20                                                          // Estimated acceleration delay of steering motors at low speeds (seconds)
#define STEERING_TOLERANCE 1.0                                                                  // Steering tolerance in degrees
#define STEERING_STALL_DETECT_ANGULAR_SPEED 5.0                                                 // Detect steering motor stall if measured angular speed is below this
#define STEERING_SLOW_APPROACH_SPEED (0.16 * (MOTOR_MAX_POWER / STEERING_MOTOR_SPEED_LIMIT))    // Slow approach speed for steering motors
#define STEERING_TOLERANCE_DISABLE_DRIVE 30.0                                                    // Disable the drive motors if any steering motor is off-target by more than this many degrees
#define STEERING_HOVER_RANGE 10.0                                                               // Angular range where steering motors tend to hover around their targets

// Claw status
#define CLAW_UNKNOWN        1   // Position unknown
#define CLAW_STOPPED        2   // Stopped in current position (somewhere between open and closed)
#define CLAW_CLOSED         3   // Claw is currently closed
#define CLAW_OPEN           4   // Claw is currently open
#define CLAW_MOVING         5   // The claw is currently moving

// Claw commands
#define CLAW_COMMAND_UNSET 0    // No command has been set yet, not doing anything
// #define CLAW_COMMAND_STOP   1   // Stop immediately, no matter the location
#define CLAW_COMMAND_CLOSE  2   // Close the claw
#define CLAW_COMMAND_OPEN   3   // Open the claw
#define CLAW_COMMAND_STAY   4   // Maintain the current position

// Claw things
#define CLAW_OPEN_ANGLE 90.0f       // Open position of the claw
#define CLAW_CLOSED_ANGLE -90.0f    // Closed position of the claw
#define CLAW_DEFAULT_ANGLE 0.0f     // Default starting claw position

// Tilt servo control parameters
#define TILT_DEGREES_PER_SECOND         50.0f // Speed in degrees per second that the tilt motor will move up or down (except when resetting to a flat position)
#define TILT_ANGLE_MIN_UPDATE_INTERVAL  0.2f // Update the tilt servo's target angle only after this many seconds have elapsed since the previous angle update
#define TILT_ANGLE_MIN_UPDATE_LOOPS     (TILT_ANGLE_MIN_UPDATE_INTERVAL / LOOP_DELAY_SECONDS) // Previous value expressed as a number of control loops to wait between updates
#define TILT_ANGLE_UPDATE_DISTANCE      (TILT_DEGREES_PER_SECOND * TILT_ANGLE_MIN_UPDATE_INTERVAL) // Distance in degrees to shift the servo angle by each update
#define TILT_MAX_ANGLE                  90.0f // Maximum angle allowed for the tilt servo
#define TILT_MIN_ANGLE                  -90.0f // Minimum angle allowed for the tilt servo
#define TILT_FLAT_ANGLE                 0.0f // Default/flat/starting angle for the tilt servo

// Tilt servo commands
#define TILT_COMMAND_UNSET  0   // Command not yet set, go to default (flat) position
#define TILT_COMMAND_RESET  1   // Reset the tilt servo to the flat position
#define TILT_COMMAND_UP     2   // Raise the tilt servo
#define TILT_COMMAND_DOWN   3   // Lower the silt servo

// Control parameters, specify which buttons control new_angle actions
#define ARM_UP_BUTTON       DPAD_UP
#define ARM_DOWN_BUTTON     DPAD_DOWN
#define CLAW_OPEN_BUTTON    DPAD_LEFT
#define CLAW_CLOSE_BUTTON   DPAD_RIGHT
#define TILT_UP_BUTTON      DIAMOND_UP
#define TILT_DOWN_BUTTON    DIAMOND_DOWN
#define TILT_RESET_BUTTON   DIAMOND_LEFT



// Button definitions
#define SerComm Serial1   //Serial port connected to Xbee
#define DIAMOND_LEFT 0
#define DIAMOND_DOWN 1
#define DIAMOND_RIGHT 2
#define DIAMOND_UP 3
#define SHOULDER_TOP_LEFT 4
#define SHOULDER_TOP_RIGHT 5
#define SHOULDER_BOTTOM_LEFT 6
#define SHOULDER_BOTTOM_RIGHT 7
#define SMALL_LEFT 8
#define SMALL_RIGHT 9
//10 and 11 are probably the stick buttons
//but we haven't checked recently
#define DPAD_UP 12
#define DPAD_RIGHT 13
#define DPAD_DOWN 14
#define DPAD_LEFT 15

// pins for motor controller 1 (right)
#define ALI1 0
#define AHI1 1
#define BHI1 2
#define BLI1 3
#define DENABLE1 8
//#define DREADY1 30

// and 2 (left)
#define ALI2 4
#define AHI2 5
#define BHI2 6
#define BLI2 7
#define DENABLE2 9
//#define DREADY2 31

#define try_enable_right(e,VBATT) try_enable_osmc(e,DENABLE1,VBATT,ALI1,BLI1,AHI1,BHI1)
#define try_enable_left(e,VBATT) try_enable_osmc(e,DENABLE2,VBATT,ALI2,BLI2,AHI2,BHI2)
#define drive_right(e,x) drive_osmc(e,DENABLE1,x,0,ALI1,BLI1,AHI1,BHI1)
#define drive_left(e,x) drive_osmc(e,DENABLE2,x,0,ALI2,BLI2,AHI2,BHI2)

#define DEADBAND_HALF_WIDTH 10  // Control input deadband radius
#define FAILTIME 200    //Failsafe timeout in milliseconds
#define DEBUGPRINT(x) SerCommDbg.println(x)
#define SerCommDbg Serial   //Serial port for debugging info
