#include <Arduino.h>
#include "globals.h"
#include "spinlock.h"

void spinlock_lock_core_0(byte* spinlock_flag) // Lock a spinlock from core 0
{
    while(*spinlock_flag == SPINLOCK_LOCK_CORE_1) {} // Wait until the data is not locked by the other core
    *spinlock_flag = SPINLOCK_LOCK_CORE_0; // Try setting the flag to lock the data
    // Wait to see if the memory was overwritten by the other core
    delay(1);
    if(*spinlock_flag == SPINLOCK_LOCK_CORE_1 || *spinlock_flag == SPINLOCK_OPEN) { // The other core locked the data (and currently holds a lock or has released it), recurse to try again
        spinlock_lock_core_0(spinlock_flag);
        return;
    }
    return;
}

void spinlock_lock_core_1(byte* spinlock_flag) // Lock a spinlock from core 1
{
    while(*spinlock_flag == SPINLOCK_LOCK_CORE_1) {} // Wait until the data is not locked by the other core
    *spinlock_flag = SPINLOCK_LOCK_CORE_0; // Try setting the flag to lock the data
    // Wait to see if the memory was overwritten by the other core
    delay(1);
    if(*spinlock_flag == SPINLOCK_LOCK_CORE_1 || *spinlock_flag == SPINLOCK_OPEN) { // The other core locked the data (and currently holds a lock or has released it), recurse to try again
        spinlock_lock_core_0(spinlock_flag);
        return;
    }
    return;
}

void spinlock_release(byte* spinlock_flag) // Release a spinlock
{
    *spinlock_flag = SPINLOCK_OPEN;
}
